;***************************************************************************
;PROGRAM:       man286.asm                                                 *
;AUTHOR:        Erika Schulze CIS:100775,2275                              *
;ASSEMBLER:     TASM 4.0                                                   *
;WRITTEN:       15. 04. 1996                                               *
;***************************************************************************

                .286

video           equ     0a000h          ;video segment
last_column     equ     639             ;the last column number
last_row        equ     240             ;the last row number
deep            equ     100             ;number of iterations

DGROUP          group _DATA,STACK       ;automatic data group

_DATA           segment word public 'DATA'

                                        ;real numbers
start_real      dd      0c0200000h      ;=-2.5
end_real        dd      3fc00000h       ;=1.5
start_imag      dd      0bfc00000h      ;=-1.5
end_imag        dd      3fc00000h       ;=1.5
step_real       dd      0
step_imag       dd      0

                                        ;used integers
row_num         dw      480             ;number of rows
col_num         dw      640             ;number of columns
c_real          dw      0
c_imag          dw      0
z_real          dw      0
z_imag          dw      0
q_real          dw      0
q_imag          dw      0
i               dw      0               ;counter row loop
l               dw      0               ;conter symmetrical row
j               dw      0               ;counter column loop
k               dw      0               ;iteration counter
multi           dw      512             ;multiplyer

                                        ;bytes
col             db      0               ;colour

_DATA           ends

_TEXT           segment word public 'CODE'
                assume  cs:_TEXT,ds:DGROUP,ss:STACK

;***************************************************************************
;PROCEDURE calculation                                                     *
;Task:          Computes the Mandelbrot Set (M-Set).                       *
;Input:         Nothing                                                    *
;Output:        M-Set on the screen                                        *
;Uses:          no procedures                                              *
;***************************************************************************
calculation     proc    near
                fninit                  ;initialize the FPU
                fld     end_imag        ;st(0):=end_imag
                fsub    start_imag      ;st(0):=(end_imag-start_imag)
                fidiv   row_num         ;st(0):=(end_imag-start_imag)/row_num
                fstp    step_imag       ;step_imag:=st(0)
                fld     end_real        ;st(0):=end_real
                fsub    start_real      ;st(0):=end_real-start_real
                fidiv   col_num         ;st(0):=(end_real-start_real)/col_num
                fstp    step_real       ;step_real:=st(0)
iloop:          mov     ax,480          ;ax:=480
                sub     ax,word ptr ds:i;ax:=480-i
                mov     word ptr ds:l,ax;l:=ax
jloop:          fild    i               ;st(0):=i
                fmul    step_imag       ;st(0):=step_imag*i
                fadd    start_imag      ;st(0):=step_imag*i+start_imag
                fimul   multi           ;st(0):=multi*st(0)
                frndint                 ;st(0):=INTEGER(st(0))
                fistp   c_imag          ;c_imag:=st(0)
                fild    j               ;st(0):=j
                fmul    step_real       ;st(0):=step_real*j
                fadd    start_real      ;st(0):=step_real*j+start_real
                fimul   multi           ;st(0):=multi*st(0)
                frndint                 ;st(0):=INTEGER(st(0))
                fistp   c_real          ;c_real:=st(0)
                fwait                   ;wait
                mov     ax,c_imag       ;ax:=c_imag
                mov     bp,ax           ;bp:=c_imag
                mov     si,ax           ;si:=z_imag
                mov     bx,c_real       ;bx:=c_real
                mov     di,bx           ;di:=z_real
                mov     ax,di           ;ax:=z_real
                imul    ax              ;dx:ax:=z_real*z_real
                shr     dx,1            ;1 shift right
                rcr     ax,1
                xchg    ah,al           ;8 shifts right
                mov     ah,dl           ;ax:=z_real*z_real/multi
                mov     word ptr q_real,ax
                                        ;q_real:=ax
                mov     ax,si           ;ax:=z_imag
                imul    ax              ;dx:ax:=z_imag*z_imag
                shr     dx,1
                rcr     ax,1
                xchg    ah,al
                mov     ah,dl           ;ax:=z_imag*z_imag/multi
                mov     word ptr q_imag,ax
                                        ;q_imag:=ax
kloop:          inc     word ptr ds:k   ;increment iteration counter
                xor     al,al           ;al:=0=black
                mov     byte ptr ds:col,al
                                        ;col:=al
                mov     ax,di           ;ax:=z_real
                imul    si              ;dx:ax:=z_real*z_imag
                xchg    ah,al
                mov     ah,dl           ;ax:=2*z_real*z_imag
                add     ax,bp           ;ax:=2*z_real*z_imag+c_imag
                mov     si,ax           ;si:=z_imag
                mov     ax,q_real       ;ax:=q_real
                sub     ax,word ptr ds:q_imag
                                        ;ax:=q_real-q_imag
                add     ax,bx           ;ax:=q_real-q_imag+c_real
                mov     di,ax           ;di:=z_real
                imul    ax              ;dx:ax:=z_real*z_real
                shr     dx,1
                rcr     ax,1
                xchg    ah,al
                mov     ah,dl           ;ax:=z_real*z_real/multi
                mov     word ptr ds:q_real,ax
                mov     ax,si           ;ax:=z_imag
                imul    ax              ;dx:ax:=z_imag*z_imag
                shr     dx,1
                rcr     ax,1
                xchg    ah,al
                mov     ah,dl           ;ax:=z_imag*z_imag/multi
                mov     word ptr ds:q_imag,ax
                                        ;q_imag:=ax (for next iteration)
                add     ax,word ptr ds:q_real
                                        ;ax:=q_real+q_imag
                shr     ax,1            ;fast division
                xchg    ah,al
                cbw                     ;ax:=(q_real+q_imag)/multi
                cmp     ax,4            ;bailout test
                jl      calculation1    ;no: test iteration counter
                mov     ax,k            ;yes: get iteration counter
                and     ax,000fh        ;ax:=k MOD 16
                mov     byte ptr ds:col,al
                                        ;set colour
                jmp     short calculation2
                                        ;go and set the pixels
calculation1:   mov     ax,k            ;get iteration counter
                cmp     ax,deep         ;loop end?
                jg      calculation2    ;yes: go and set the pixels
                jmp     short kloop     ;no: next iteration
calculation2:   mov     word ptr ds:k,0 ;set k=0
                mov     dx,i            ;dx:=row=y
                mov     ax,80
                mul     dx              ;ax:=y*Bytes per row
                mov     bx,j            ;bx:=column=x
                mov     cx,bx           ;save it in cx
                shr     bx,3            ;bx:=x/8
                add     bx,ax           ;bx:=offset=y*Bytes per row+x/8
                and     cx,0007h        ;cx:=x MOD 8
                xor     cl,7            ;cx:=7-x MOD 8
                mov     ah,1            ;generate bit mask
                shl     ah,cl           ;in ah
                mov     dx,3ceh         ;dx:=port address adapter
                mov     al,8            ;al:=address Bit-Mask-Register
                out     dx,ax
                mov     ax,0205h        ;write mode 2
                out     dx,ax
                mov     ax,video
                mov     es,ax           ;es:=segment video RAM
                mov     al,es:[bx]      ;fill the latches
                                        ;value in al is unimportant
                mov     al,col          ;al:=colour
                mov     byte ptr es:[bx],al
                                        ;set 1. pixel
                mov     dx,l
                mov     ax,80
                mul     dx
                mov     bx,j
                shr     bx,3
                add     bx,ax
                mov     al,es:[bx]
                mov     al,col
                mov     byte ptr es:[bx],al
                                        ;set 2. pixel
                mov     ax,j            ;get column counter
                cmp     ax,last_column  ;all columns done?
                jge     calculation3    ;yes: go and test the rows
                inc     ax              ;no: increment column counter
                mov     word ptr ds:j,ax
                jmp     jloop           ;next column
calculation3:   mov     word ptr ds:j,0 ;set j=0
                mov     ax,i            ;get row counter
                cmp     ax,last_row     ;all rows done?
                jge     calculation4    ;yes: the job is done
                inc     ax              ;no: increment row counter
                mov     word ptr ds:i,ax
                jmp     iloop           ;next row
calculation4:   ret
calculation     endp
;---------------------------------------------------------------------------

;***************************************************************************
;PROCEDURE waiting                                                         *
;Task:          Waits until the user has pressed a key.                    *
;Input:         Nothing                                                    *
;Output:        waiting                                                    *
;Uses:          INT 21H of DOS                                             *
;***************************************************************************
waiting         proc    near
                mov     ah,8h           ;function: wait for key
                int     21h             ;transfer to DOS
                ret
waiting         endp
;---------------------------------------------------------------------------

;***************************************************************************
;PROCEDURE textmode                                                        *
;Task:          Sets the text mode.                                        *
;Input:         Nothing                                                    *
;Output:        text mode                                                  *
;Uses:          INT 10H of the BIOS                                        *
;***************************************************************************
textmode        proc    near
                mov     ax,0003h        ;set text mode
                int     10h             ;transfer to BIOS
                ret
textmode        endp
;---------------------------------------------------------------------------

;***************************************************************************
;PROCEDURE screen12                                                        *
;Task:          Sets the mode 12H (VGA 640*480*16)                         *
;Input:         Nothing                                                    *
;Output:        Mode 12H                                                   *
;Uses:          INT 10H of the BIOS                                        *
;***************************************************************************
screen12        proc    near
                mov     ax,0012h        ;VGA (640*480*16)
                int     10h             ;transfer to BIOS
                ret
screen12        endp
;---------------------------------------------------------------------------

;***************************************************************************
;MAIN PROCEDURE mandel                                                     *
;***************************************************************************
mandel          proc    far
                pusha                   ;save all registers
                mov     ax,seg DGROUP   ;make datas adressable
                mov     ds,ax           ;via ds
                call    screen12        ;set VGA mode
                call    calculation     ;compute the M-Set
                call    waiting         ;wait for key
                call    textmode        ;set text mode
                popa                    ;restore all registers
                mov     ax,4c00h        ;load return code
                int     21h             ;transfer to DOS
mandel          endp
;===========================================================================
;END MAIN PROCEDURE                                                        =
;===========================================================================

_TEXT           ends

STACK           segment para stack 'STACK'

                db      128 dup(?)

STACK           ends
                end     mandel
